﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Crm4ScriptExtractor.Model;
using Crm4ScriptExtractor.Util;

namespace Crm4ScriptExtractor.DataAccess
{
    class CSVScriptWriter : IScriptDataWriter
    {
        #region コンストラクタ
        public CSVScriptWriter(string filename)
        {
            _filename = filename;
        }
        #endregion

        #region IScriptDataWriter Members

        public void WriteData(ScriptDataModel model)
        {
            using (CSVWriter writer = new CSVWriter(_filename, Encoding.UTF8))
            {
                writer.IsTokenEnclosedWithDQuotas = true;
                writer.Separator = ",";

                // outpout header
                writer.WriteLine(ToHeaderToken(model.Header));
                model.Rows.ForEach(delegate(RowDataModel row) { writer.WriteLine(ToRowToken(row)); });
            }
        }

        private string _filename;

        public string[] ToHeaderToken(RowHeaderModel header)
        {
            List<string> s = new List<string>();
            s.Add(header.EntityLogicalName);
            s.Add(header.EntityDisplayName);
            s.Add(header.AttributeLogicalName);
            s.Add(header.AttributeDisplayName);
            s.Add(header.Event);
            s.Add(header.Script);

            return s.ToArray();
        }
        public string[] ToRowToken(RowDataModel row)
        {
            List<string> s = new List<string>();
            s.Add(row.EntityLogicalName);
            s.Add(row.EntityDisplayName);
            s.Add(row.AttributeLogicalName);
            s.Add(row.AttributeDisplayName);
            s.Add(row.EventType.ToString());
            s.Add(row.Script);

            return s.ToArray();
        }

        #endregion
    }
}
