﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Crm4ScriptExtractor.Model
{
    /// <summary>
    /// Javascript の1イベントを表すモデル
    /// </summary>
    public class RowDataModel
    {

        private string _entityLogicalName;
        /// <summary>
        /// エンティティの論理名
        /// </summary>
        public string EntityLogicalName
        {
            get { return _entityLogicalName; }
            set { _entityLogicalName = value; }
        }
        private string _entityDisplayName;
        /// <summary>
        /// エンティティの表示名
        /// </summary>
        public string EntityDisplayName
        {
            get { return _entityDisplayName; }
            set { _entityDisplayName = value; }
        }
        private string _attributeLogicalName;
        /// <summary>
        /// 属性の論理名
        /// </summary>
        public string AttributeLogicalName
        {
            get { return _attributeLogicalName; }
            set { _attributeLogicalName = value; }
        }
        private string _attributeDisplayName;
        /// <summary>
        /// 属性の表示名
        /// </summary>
        public string AttributeDisplayName
        {
            get { return _attributeDisplayName; }
            set { _attributeDisplayName = value; }
        }
        private ScriptEventType _eventType;
        /// <summary>
        /// イベント種別
        /// </summary>
        public ScriptEventType EventType
        {
            get { return _eventType; }
            set { _eventType = value; }
        }
        private string _script;
        /// <summary>
        /// スクリプト本体
        /// </summary>
        public string Script
        {
            get { return _script; }
            set { _script = value; }
        }
        public static RowDataModel CreateOnLoad(string entityLogicalName, string entityDisplayName, string script)
        {
            RowDataModel model = new RowDataModel();
            model.EntityLogicalName = entityLogicalName;
            model.EntityDisplayName = entityDisplayName;
            model.AttributeLogicalName = "-";
            model.AttributeDisplayName = "-";
            model.EventType = ScriptEventType.OnLoad;
            model.Script = script;

            return model;
        }
        public static RowDataModel CreateOnSave(string entityLogicalName, string entityDisplayName, string script)
        {
            RowDataModel model = new RowDataModel();
            model.EntityLogicalName = entityLogicalName;
            model.EntityDisplayName = entityDisplayName;
            model.AttributeLogicalName = "-";
            model.AttributeDisplayName = "-";
            model.EventType = ScriptEventType.OnSave;
            model.Script = script;
            return model;
        }
        public static RowDataModel CreateOnChange(string entityLogicalName, string entityDisplayName
            , string attributeLogicalName, string attributeDisplayName, string script)
        {
            RowDataModel model = new RowDataModel();
            model.EntityLogicalName = entityLogicalName;
            model.EntityDisplayName = entityDisplayName;
            model.AttributeLogicalName = attributeLogicalName;
            model.AttributeDisplayName = attributeDisplayName;
            model.EventType = ScriptEventType.OnChange;
            model.Script = script;
            return model;
        }
    }
    /// <summary>
    /// RowDataModelのコレクション
    /// </summary>
    public class RowDataModelCollection : List<RowDataModel>
    {
        public void AddOnSave(string entityLogicalName, string entityDisplayName, string script)
        {
            RowDataModel item = RowDataModel.CreateOnSave(entityLogicalName, entityDisplayName, script);
            this.Add(item);
        }
        public void AddOnLoad(string entityLogicalName, string entityDisplayName, string script)
        {
            RowDataModel item = RowDataModel.CreateOnLoad(entityLogicalName, entityDisplayName, script);
            this.Add(item);
        }
        public void AddOnChange(string entityLogicalName, string entityDisplayName
            , string attributeLogicalName, string attributeDisplayName, string script)
        {
            RowDataModel item = RowDataModel.CreateOnChange(entityLogicalName, entityDisplayName
                , attributeLogicalName, attributeDisplayName, script);
            this.Add(item);
        }
    }
}
