﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Crm4ScriptExtractor.Model;

namespace Crm4ScriptExtractor.Service
{
    /// <summary>
    /// カスタマイゼーションファイルからエンティティのスクリプト情報を抽出するクラス
    /// </summary>
    class ExtractScriptService
    {
        private const string ONLOAD = "onload";
        private const string ONSAVE = "onsave";
        private const string ONCHANGE = "onchange";

        public ScriptDataModel ExtractScripts(string filename)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(filename);

            RowDataModelCollection rows = ProcessDocument(doc);
            RowHeaderModel header = RowHeaderModel.Create();
            ScriptDataModel model = new ScriptDataModel(header, rows);

            return model;
        }

        public RowDataModelCollection ProcessDocument(XmlDocument doc)
        {
            RowDataModelCollection models = new RowDataModelCollection();

            XmlNode relationships = doc.SelectSingleNode("/ImportExportXml/EntityRelationships");

            foreach (XmlNode element in doc.SelectNodes("/ImportExportXml/Entities/Entity"))
            {
                ProcessEntity(element, relationships, models);
            }

            return models;
        }
        public void ProcessEntity(XmlNode entity, XmlNode relationships, RowDataModelCollection models)
        {
            XmlNode nameNode = entity.SelectSingleNode("Name");
            string logicalName = nameNode.InnerText;
            string displayname = nameNode.Attributes["LocalizedName"].Value;

            foreach (XmlNode element in entity.SelectNodes(".//events/event[@application = 'false' and @active='true']"))
            {
                // イベント名
                string eventName = element.Attributes["name"].Value;
                // スクリプト
                string script = element.SelectSingleNode("script").InnerText;

                if (eventName.Equals(ONCHANGE, StringComparison.InvariantCultureIgnoreCase))
                {
                    XmlNode control = element.ParentNode.ParentNode.SelectSingleNode("control");
                    string field = control.Attributes["datafieldname"].Value;
                    // translate(@targetAttribute, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz')
                    string query = string.Format("EntityInfo/entity/attributes/attribute[translate(@PhysicalName,'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz') ='{0}']", field);
                    XmlNode attribute = entity.SelectSingleNode(query);
                    if (attribute == null)
                    {
                        // Relationship 検索
                        //query = string.Format("EntityRelationship[ReferencingEntityName/text() = '{0}' and ReferencingAttributeName/text() = '{1}']", logicalName, field);
                        //query = string.Format("EntityRelationship[ReferencingEntityName/text() = '{0}' ]", logicalName);
                        query = string.Format("EntityRelationship/field[@name = '{0}' and ../ReferencingEntityName = '{1}']", field, logicalName);
                        attribute = relationships.SelectSingleNode(query);
                    }
                    XmlNode node = attribute.SelectSingleNode("displaynames/displayname");
                    string attributename = node.Attributes["description"].Value;

                    models.AddOnChange(logicalName, displayname, field, attributename, script);
                }
                else if (eventName.Equals(ONLOAD, StringComparison.InvariantCultureIgnoreCase))
                {
                    models.AddOnLoad(logicalName, displayname, script);
                }
                else if(eventName.Equals(ONSAVE, StringComparison.InvariantCultureIgnoreCase))
                {
                    models.AddOnSave(logicalName, displayname, script);
                }
            }
            
        }

    }
}
